'use strict';

Object.defineProperty(exports, '__esModule', {
  value: true
});

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _videoJs = require('video.js');

var _videoJs2 = _interopRequireDefault(_videoJs);

// Default options for the plugin.
var defaults = {
  language: { direction: 'ltr' },
  captions: []
};

// the captions being currently displayed.
var settings = {
  display: []
};

/**
 * Supported formatting styles 'NAME': ['startTag', 'endTag']
 */
var styleToTag = {
  BOLD: ['<b>', '</b>'],
  ITALIC: ['<i>', '</i>'],
  UNDERLINE: ['<u>', '</u>']
};

/**
 * Function is used to convert a caption object to renderable HTML.
 *
 * @function convertToHtml
 * @param    {Array} [caption={Caption}]
 */
var convertToHtml = function convertToHtml(caption) {
  var displayValue = '';
  var str = caption.content;

  if (caption.inlineStyles) {
    for (var i = 0; i <= str.length; i++) {
      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = caption.inlineStyles[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var style = _step.value;

          if (i === style.offset) {
            displayValue += styleToTag[style.style][0];
          } else if (i === style.offset + style.length) {
            displayValue += styleToTag[style.style][1];
          }
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator['return']) {
            _iterator['return']();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      if (i < str.length) {
        displayValue += str[i];
      }
    }
  } else {
    displayValue = caption.content;
  }
  return displayValue.replace(/\n/g, '<br/>');
};

/**
 * Creates a container that holds caption. Uses the position of the passed in
 * caption as the location for the container.
 *
 * @function createContainer
 * @param    {Object} caption
 * @param    {Player} player
 */
var createContainer = function createContainer(caption, player) {
  var container = document.createElement('div');

  if (settings.language && settings.language.direction) {
    container.dir = settings.language.direction;
  }
  container.classList.add('vjs-caption-containter');
  // set the horizontal position
  if (caption.horizontalPosition) {
    container.style['text-align'] = caption.horizontalPosition.toLowerCase();
  } else {
    container.style['text-align'] = 'center';
  }

  // set the vertical position
  if (caption.verticalPosition) {
    var vp = caption.verticalPosition;

    if (vp === 'BOTTOM') {
      container.style.bottom = '40px';
      container.style.top = '';
    } else if (vp === 'MIDDLE') {
      container.style.bottom = '';
      container.style.top = player.height() / 3 + 10 + 'px';
    } else if (vp === 'TOP') {
      container.style.bottom = '';
      container.style.top = '10px';
    }
  } else {
    container.bottom = '40px';
    container.style.top = '';
  }

  return container;
};

/**
 * Renders a caption into the player/container.
 *
 * @function renderCaption
 * @param    {Object} caption
 * @param    {Player} player
 * @param    {DOMElement} container
 */
var renderCaption = function renderCaption(caption, player, container) {
  var videoEl = player.el();
  var wrapper = document.createElement('div');
  var span = document.createElement('div');

  container.appendChild(wrapper);
  wrapper.appendChild(span);
  wrapper.classList.add('vjs-caption');
  span.classList.add('vjs-text');
  videoEl.appendChild(container);

  span.innerHTML = convertToHtml(caption);
};

/**
 * Renders the captions that should be on screen.
 *
 * @function updateCaption
 * @param    {Player} player
 */
var updateCaption = function updateCaption(player) {
  var time = player.currentTime() * 1000;

  var videoEl = player.el();
  var captions = settings.captions.filter(function (caption) {
    return caption.start <= time && caption.end >= time;
  });

  if (!(JSON.stringify(captions) === JSON.stringify(settings.display))) {

    // clear the existing content
    var oldCaptionContainers = [].slice.call(videoEl.getElementsByClassName('vjs-caption-containter'));

    oldCaptionContainers.forEach(function (e) {
      return e.parentNode.removeChild(e);
    });

    // keep a list of created containers. We can reuse ones for captions in the same spot.
    var containers = {};

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
      for (var _iterator2 = captions[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
        var caption = _step2.value;

        var container = undefined;

        if (containers[caption.horizontalPosition + '-' + caption.verticalPosition]) {
          // use the existing div
          container = containers[caption.horizontalPosition + '-' + caption.verticalPosition];
        } else {
          // create a new div
          container = createContainer(caption, player);
          containers[caption.horizontalPosition + '-' + caption.verticalPosition] = container;
        }

        renderCaption(caption, player, container);
        settings.display = captions;
      }
    } catch (err) {
      _didIteratorError2 = true;
      _iteratorError2 = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion2 && _iterator2['return']) {
          _iterator2['return']();
        }
      } finally {
        if (_didIteratorError2) {
          throw _iteratorError2;
        }
      }
    }
  }
};

/**
 * Function is used to set the captions for the plugin. Captions must be an array.
 *
 * Captions from v2 of Dotsub are in the form:
 *          {start: timeInMills, duration: timeInMills, content: 'Caption Text'}
 * Captions from v3 of Dotsub are in the form:
 *          {start: timeInMills, end: timeInMills, content: 'Caption Text'}
 * v3 can also contain formatting and position data {position, inlineStyles: []}
 *
 * @function setCaptions
 * @param    {Player} player
 * @param    {Array} [captions=[{Caption}]]
 */
var setCaptions = function setCaptions(player, captions) {
  settings.captions = captions;
  updateCaption(player);
};

/**
 * Function is used to set the language of the video player.
 *
 * Expects { direction: 'rtl' or 'ltr' }
 *
 * @function setLanguage
 * @param    {Player} player
 * @param    {Object} [language={}]
 */
var setLanguage = function setLanguage(player, language) {
  settings.language = language;
};

/**
 * Sets up HTML in the player for caption rendering.
 *
 * Expects { direction: 'rtl' or 'ltr' }
 *
 * @function setupCaptions
 * @param    {Player} player
 */
var setupCaptions = function setupCaptions(player) {
  // export functions to be called externally
  player.setCaptions = function (captions) {
    return setCaptions(player, captions);
  };
  player.setLanguage = function (langauge) {
    return setLanguage(player, langauge);
  };
  // setup event listeners a well.
  player.on('captions', function (event, data) {
    return setCaptions(player, data);
  });
  player.on('language', function (event, data) {
    return setLanguage(player, data);
  });
  player.on('timeupdate', function () {
    return updateCaption(player);
  });
  // update the view using the initial captions
  updateCaption(player);
};

/**
 * Function to invoke when the player is ready.
 *
 * This is a great place for your plugin to initialize itself. When this
 * function is called, the player will have its DOM and child components
 * in place.
 *
 * @function onPlayerReady
 * @param    {Player} player
 * @param    {Object} [options={}]
 */
var onPlayerReady = function onPlayerReady(player, options) {
  player.addClass('vjs-dotsub-captions');

  setupCaptions(player);

  // emit that the player is ready to receive captions
  player.trigger('captionsready');
};

/**
 * A video.js plugin.
 *
 * In the plugin function, the value of `this` is a video.js `Player`
 * instance. You cannot rely on the player being in a "ready" state here,
 * depending on how the plugin is invoked. This may or may not be important
 * to you; if not, remove the wait for "ready"!
 *
 * @function dotsubCaptions
 * @param    {Object} [options={}]
 *           An object of options left to the plugin author to define.
 */
var dotsubCaptions = function dotsubCaptions(options) {
  var _this = this;

  this.ready(function () {
    settings = _videoJs2['default'].mergeOptions(defaults, options);
    onPlayerReady(_this, settings);
  });
};

// Register the plugin with video.js.
_videoJs2['default'].plugin('dotsubCaptions', dotsubCaptions);

// Include the version number.
dotsubCaptions.VERSION = '__VERSION__';

exports['default'] = dotsubCaptions;
module.exports = exports['default'];